searchNodes=[{"doc":"Lamina allows you to define a run-time configuration pipeline that can merge configuration from several sources. This allows the system to be reactive to changes in its environment. Example The following example defines a configuration for an imaginary HTTP server application which takes it's configuration from a combination of default values, the OTP application environment and system environment variables: defmodule MyHttpServer.Config do use Lamina provider ( Lamina.Provider.Default , listen_port : 4000 , listen_address : &quot;0.0.0.0&quot; ) provider ( Lamina.Provider.ApplicationEnv , otp_app : :my_http_server , key : MyHttpServer.Endpoint ) provider ( Lamina.Provider.Env , prefix : &quot;HTTP&quot; ) config :listen_port do cast ( &amp; Lamina.Cast . to_integer / 1 ) validate ( fn port when is_integer ( port ) and ( port in [ 80 , 443 ] or port &gt;= 1000 ) -&gt; true _ -&gt; false end ) end config :listen_address do validate ( fn address when is_binary ( address ) -&gt; address |&gt; String . to_charlist ( ) |&gt; :inet . parse_address ( ) |&gt; case do { :ok , _ } -&gt; true _ -&gt; false end _ -&gt; false end ) end end Provider order is preserved, such that providers added later (via the provider/1 or provider/2 macro) have more priority than their predecessors. This has the effect that when more than one provider can provide a value for a given configuration item, the most preferred value will be returned. Each configuration item is defined using the config/1 or config/2 macro. If the configuration item does not need casting to another type, nor validation then just defining it with config/1 is sufficient. In some cases it is necessary to provide additional casting or validating functions. They can be provided by passing a block containing the cast/1 or validate/1 macros. Make sure that you add your configuration module to your application's supervisor tree before any processes that rely on it's information. Lamina will fail to start or shutdown on any errors it encounters. Lifetimes All configuration items in Lamina are explicitly marked with a lifetime, which must be specified by the configuration provider when returning values. The semantics are as follows: :volatile - a configuration that could potentially be different every time it is read. Volatile configuration items are returned by the ApplicationEnv and Env providers. :static - a configuration value that is not going to change until the provider changes it. Static configuration items are returned by the Default provider, but could also be used for a configuration provider that notifies the system of configuration changes in some way. {non_neg_integer(), System.time_unit()} - a value that has a specific expiry time. This may be used for a configuration source that has explicit leases on values (ala Vault or a value for which querying is expensive, and providing an expiry would effectively cache it. Querying When asked to retrieve a configuration value, Lamina queries it's ETS table using the following query plan; values for which there is no expiry, or which have not yet expired, ordered by provider weight, descending. It only ever returns a single row. If the returned row is marked as :volatile then the configuration provider is immediately queried for a new value, meaning that these requests will pay the cost of a GenServer.call/3 to ensure freshness. If this is an issue then you should consider changing the provider lifetime to use an expiry. The ApplicationEnv and Env providers have a configuration option to do this. If you are the developer of a volatile provider, it is strongly suggested that you provide for this use case. Server configuration The following options can be passed to the use Lamina macro, although it's probably advisable to leave them as their defaults. gc_timeout: pos_integer() - how long the server should be idle before removing expired configuration from the ETS table in milliseconds. Defaults to 3000. ttl_refresh_fraction: float - when presented with a configuration value which has an expiry, the server queues a refresh at some point prior to the value expiring, in order to avoid having missing configuration. Setting this to a value between 0 and 1 specifies the proportion of the expiry time to wait before attempting to refresh the value. Defaults to 0.95 . Configuration subscriptions Lamina defines a subscribe/1 and unsubscribe/1 function on each configuration module, which uses a Registry to handle pub-sub for configuration changes. This allows your processes to subscribe to configuration changes and update or restart any services they provide. Example For example, a simple HTTP server which changes it's listen port in response to a configuration change: defmodule MyHttpServer.Cowboy do use GenServer alias Plug.Cowboy alias MyHttpServer . { Config , Plug } def init ( _ ) do with { :ok , port } &lt;- Config . listen_port ( ) , { :ok , srv } &lt;- Cowboy . http ( Plug , [ ] , port : port ) , :ok &lt;- Config . subscribe ( :listen_port ) do { :ok , %{ srv : srv , port : port } } end end def handle_info ( { :config_change , Config , :listen_port , _old_port , new_port } , %{ port : current_port } ) when new_port != current_port do with :ok &lt;- Cowboy . shutdown ( Plug.HTTP ) , { :ok , srv } &lt;- Cowboy . http ( Plug , [ ] , port : new_port ) do { :noreply , %{ port : new_port , srv : srv } } else { :error , reason } -&gt; { :stop , reason , nil } end end def handle_info ( { :config_change , _ , _ , _ } , state ) , do : { :noreply , state } end","ref":"Lamina.html","title":"Lamina","type":"behaviour"},{"doc":"","ref":"Lamina.html#c:__lamina__/1","title":"Lamina.__lamina__/1","type":"callback"},{"doc":"Called when the Lamina.Server detects a configuration change. This is a simple mechanism to allow the server to notify your system that a configuration value has changed. See also subscribe/1 .","ref":"Lamina.html#c:config_change/3","title":"Lamina.config_change/3","type":"callback"},{"doc":"A helpful library of casts. Defining a cast function is brain-dead simple, however sometimes you just don't want to write the same thing over and over. Here is a helpful pile of functions for common configuration castings. Feel free to open a PR to add more.","ref":"Lamina.Cast.html","title":"Lamina.Cast","type":"module"},{"doc":"Attempts to convert the inbound value into an float.","ref":"Lamina.Cast.html#to_float/1","title":"Lamina.Cast.to_float/1","type":"function"},{"doc":"Attempts to convert the inbound value into an integer.","ref":"Lamina.Cast.html#to_integer/1","title":"Lamina.Cast.to_integer/1","type":"function"},{"doc":"Defines the macros used for building a configuration module.","ref":"Lamina.DSL.html","title":"Lamina.DSL","type":"module"},{"doc":"Defines an individual configration parameter. The same as config/2 , except that no block is provided.","ref":"Lamina.DSL.html#config/1","title":"Lamina.DSL.config/1","type":"macro"},{"doc":"Defines an individual configration parameter. Arguments: config_key - the name of the new configuration parameter to define. block - a &quot;do block&quot; which will be evaluated in the context of the Lamina.DSL.Config module. Example config :listen_port do cast ( &amp; Lamina.Cast . to_integer / 1 ) validate ( &amp; is_integer / 1 ) end","ref":"Lamina.DSL.html#config/2","title":"Lamina.DSL.config/2","type":"macro"},{"doc":"Defines a provider for the configuration system. The same as provider/2 , however passes an empty list to the provider's init/1 function.","ref":"Lamina.DSL.html#provider/1","title":"Lamina.DSL.provider/1","type":"macro"},{"doc":"Defines a provider for the configuration system. Arguments: module - the name of a module which implements the Lamina.Provider behaviour. options - a keyword list of options to be passed to the provider's init/1 function. Example provider ( Lamina.Provider.Env , prefix : &quot;HTTP&quot; )","ref":"Lamina.DSL.html#provider/2","title":"Lamina.DSL.provider/2","type":"macro"},{"doc":"Defines the macros used inside the config macro.","ref":"Lamina.DSL.Config.html","title":"Lamina.DSL.Config","type":"module"},{"doc":"Specify a transformation function to cast the value to the final required type. Some configuration providers (most notably Env ) are only able to return strings, so it can be necessary to modify them before they're returned to the user. Example defmodule MyHttpServer.Config do use Lamina provider ( Lamina.Provider.Env ) config :listen_port do cast ( &amp; String . to_integer / 1 ) end end","ref":"Lamina.DSL.Config.html#cast/1","title":"Lamina.DSL.Config.cast/1","type":"macro"},{"doc":"Specify a validation function to ensure that the value is valid. Gives you an opportunity to ensure that the value about to be returned to the user is correct. Example defmodule MyFileReader.Config do use Lamina provider ( Lamina.Provider.Env ) config :file_to_read do validate ( fn &quot;/etc/password&quot; -&gt; false _ -&gt; true end ) end end","ref":"Lamina.DSL.Config.html#validate/1","title":"Lamina.DSL.Config.validate/1","type":"macro"},{"doc":"The Provider behaviour is used to declare configuration providers. Lamina's flexibility comes from the ability to provide adapters for may configuration sources and be able to query them in order to build a composite view of the system's configuration. The Lamina.Provider behaviour allows you to define these adapters. Example: As a minimum you must implement the fetch_config/2 callback, which is used to look up individual configuration values in your storage: defmodule MapProvider do use Lamina.Provider @impl true def fetch_config ( config_key , state ) do case Map . fetch ( state , config_key ) do { :ok , value } -&gt; { :ok , value , :volatile , state } :error -&gt; { :ok , state } end end end","ref":"Lamina.Provider.html","title":"Lamina.Provider","type":"behaviour"},{"doc":"Config change callback. Optional. Depending on the semantics of the provider, it may be necessary for the provider to proactively notify the Lamina server that a configuration value has changed. After initialisation the server will call this callback on all providers with a function as it's first argument. Calling this function will cause the server to immediately call the fetch_config/2 callback.","ref":"Lamina.Provider.html#c:config_change/2","title":"Lamina.Provider.config_change/2","type":"callback"},{"doc":"Called by Lamina when it wants to retrieve a configuration value. Arguments config_key - the name of the configuration value to be returned. state - the current provider state. Return values There are three possible return values: A successful retrieval of a configuration value, including a lifetime indicator. Inability to supply a configuration value, which is not an error. An error. Lifetime indicators When returning a configuration value, Lamina expects a provider to provide additional information about how long it can expect the value to be valid for. Valid lifetime indicators are: :static indicating that the value can never change. This is most likely for a default value, or one which may have been compiled in. :volatile indicating that the value can potentially change every time it is read. A tuple containing a combination of a positive integer and a time unit. Units are those specified by the System.time_unit typespec. For example {60, :second} or {3, :microsecond} . Useful for providers such as Vault which allow values to be leased for a specific period of time.","ref":"Lamina.Provider.html#c:fetch_config/2","title":"Lamina.Provider.fetch_config/2","type":"callback"},{"doc":"Called by Lamina when initialising a configuration provider. If you're implementing a configuration provider, then this is your opportunity to do any setup work before any configuration is requested. When returning successfully, it is possible to include a refresh_period option, which indicates how often (in milliseconds) to refresh the configuration values of this provider. This can be used by providers such as Lamina.Provider.Env which represent volatile data, but have no way of detecting change other than polling.","ref":"Lamina.Provider.html#c:init/1","title":"Lamina.Provider.init/1","type":"callback"},{"doc":"","ref":"Lamina.Provider.html#t:lifetime/0","title":"Lamina.Provider.lifetime/0","type":"type"},{"doc":"","ref":"Lamina.Provider.html#t:state/0","title":"Lamina.Provider.state/0","type":"type"},{"doc":"Configuration provider for the OTP application environment. Allows you to use the configuration from your application enviroment (typically stored in config/config.exs et al inside your project). In order to correctly find configuration for your application you must provide the otp_app option, and possibly a second-level configuration key. This is a very simple wrapper around Application.get_all_env/1 and Application.get_env/3 . See the Elixir core documentation for more information. Examples Given the following configurations: import Config config :lamina , jigga_watts : 1.21 , time_machine : [ model : :delorean , top_speed : 88 ] We can retrieve the configurations from the environment: iex&gt; { :ok , state , _refresh_period } = ApplicationEnv . init ( otp_app : :lamina ) ...&gt; { :ok , 1.21 , :volatile , _state } = ApplicationEnv . fetch_config ( :jigga_watts , state ) iex&gt; { :ok , state , _refresh_period } = ApplicationEnv . init ( otp_app : :lamina , key : :time_machine ) ...&gt; { :ok , :delorean , :volatile , _state } = ApplicationEnv . fetch_config ( :model , state )","ref":"Lamina.Provider.ApplicationEnv.html","title":"Lamina.Provider.ApplicationEnv","type":"module"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:key_option/0","title":"Lamina.Provider.ApplicationEnv.key_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:lifetime_option/0","title":"Lamina.Provider.ApplicationEnv.lifetime_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:option/0","title":"Lamina.Provider.ApplicationEnv.option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:options/0","title":"Lamina.Provider.ApplicationEnv.options/0","type":"type"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:otp_app_option/0","title":"Lamina.Provider.ApplicationEnv.otp_app_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:refresh_period_option/0","title":"Lamina.Provider.ApplicationEnv.refresh_period_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.ApplicationEnv.html#t:state/0","title":"Lamina.Provider.ApplicationEnv.state/0","type":"type"},{"doc":"Configuration provider for static default configuration values. Values are passed in as a keyword list of options to init. Example iex&gt; { :ok , state } = Default . init ( name : &quot;Marty McFly&quot; ) ...&gt; { :ok , &quot;Marty McFly&quot; , :static , _state } = Default . fetch_config ( :name , state )","ref":"Lamina.Provider.Default.html","title":"Lamina.Provider.Default","type":"module"},{"doc":"A configuration provider for retrieving information from the UNIX process environment. Allows you to use process enviroment variables as application configuration. This is a very simple wrapper around System.get_env/2 . See the Elixir core documentation for more information. Name mangling We use a process called &quot;name mangling&quot; to convert an atom configuration key into the name of an environment variable. The default mangler ( name_mangler/2 ) converts the configuration key into UPPER_SNAKE_CASE with an optional prefix. Should you need to you can define your own mangler function and pass it as the :mangler option to this provider. Examples Given the following environment variables: Name Value PHOENIX_HTTP_PORT 4000 BIND_ADDR 0.0.0.0 We can retrieve the configurations from the environment: iex&gt; { :ok , state , _ } = Env . init ( prefix : &quot;PHOENIX&quot; ) ...&gt; { :ok , &quot;4000&quot; , :volatile , _state } = Env . fetch_config ( :http_port , state ) iex&gt; { :ok , state , _ } = Env . init ( [ ] ) ...&gt; { :ok , &quot;0.0.0.0&quot; , :volatile , _state } = Env . fetch_config ( :bind_addr , state )","ref":"Lamina.Provider.Env.html","title":"Lamina.Provider.Env","type":"module"},{"doc":"The default name mangler for environment variables. Converts the configuration key into an upper-case underscored string, optionally with a prefix. Examples: iex&gt; Env . name_mangler ( :http_port , %{ } ) &quot;HTTP_PORT&quot; iex&gt; Env . name_mangler ( :http_port , %{ prefix : &quot;phoenix&quot; } ) &quot;PHOENIX_HTTP_PORT&quot;","ref":"Lamina.Provider.Env.html#name_mangler/2","title":"Lamina.Provider.Env.name_mangler/2","type":"function"},{"doc":"","ref":"Lamina.Provider.Env.html#t:lifetime_option/0","title":"Lamina.Provider.Env.lifetime_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.Env.html#t:mangler_option/0","title":"Lamina.Provider.Env.mangler_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.Env.html#t:option/0","title":"Lamina.Provider.Env.option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.Env.html#t:options/0","title":"Lamina.Provider.Env.options/0","type":"type"},{"doc":"","ref":"Lamina.Provider.Env.html#t:prefix_option/0","title":"Lamina.Provider.Env.prefix_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.Env.html#t:refresh_period_option/0","title":"Lamina.Provider.Env.refresh_period_option/0","type":"type"},{"doc":"","ref":"Lamina.Provider.Env.html#t:state/0","title":"Lamina.Provider.Env.state/0","type":"type"},{"doc":"An Elixir Registry which keeps track of configuration subscribers.","ref":"Lamina.Registry.PubSubRegistry.html","title":"Lamina.Registry.PubSubRegistry","type":"module"},{"doc":"","ref":"Lamina.Registry.PubSubRegistry.html#publish/4","title":"Lamina.Registry.PubSubRegistry.publish/4","type":"function"},{"doc":"","ref":"Lamina.Registry.PubSubRegistry.html#subscribe/2","title":"Lamina.Registry.PubSubRegistry.subscribe/2","type":"function"},{"doc":"","ref":"Lamina.Registry.PubSubRegistry.html#unsubscribe/2","title":"Lamina.Registry.PubSubRegistry.unsubscribe/2","type":"function"},{"doc":"An Elixir Registry which keeps track of Lamina.Server processes.","ref":"Lamina.Registry.ServerRegistry.html","title":"Lamina.Registry.ServerRegistry","type":"module"},{"doc":"","ref":"Lamina.Registry.ServerRegistry.html#lookup/1","title":"Lamina.Registry.ServerRegistry.lookup/1","type":"function"},{"doc":"","ref":"Lamina.Registry.ServerRegistry.html#register/2","title":"Lamina.Registry.ServerRegistry.register/2","type":"function"},{"doc":"The Lamina configuration server. This server is the owner of the ETS table used to store the configuration information and handles providers and their states as well as timers for refreshing and expiring the configuration.","ref":"Lamina.Server.html","title":"Lamina.Server","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Lamina.Server.html#child_spec/1","title":"Lamina.Server.child_spec/1","type":"function"},{"doc":"Retrieve a configuration value. This is the function which is called by the dynamically generated configuration functions in the configuration module, ie calling MyHttpServer.Config.listen_port() and Lamina.Server.get(MyHttpServer.Config, :listen_port) are functionally identical. Looks the server pid and ETS table up in Lamina registry, then queries the table for the best match configuration value. In most cases this should allow the configuration value to be returned quickly without having to wait for a GenServer round-trip as both Registry and the server use ETS tables with read concurrency enabled. The only case where a call needs to be made to the server is when the best configuration value is marked as :volatile by the provider - ie the value can theoretically be different each time it's called. This is the case for the Env and ApplicationEnv providers, as both of these provide wrappers around a potentially volatile store. Example iex&gt; Server . get ( MyHttpServer.Config , :listen_port ) ...&gt; { :ok , 4000 }","ref":"Lamina.Server.html#get/2","title":"Lamina.Server.get/2","type":"function"},{"doc":"A raising version of get/2 .","ref":"Lamina.Server.html#get!/2","title":"Lamina.Server.get!/2","type":"function"},{"doc":"Initialise the configuration server. Initialises the configuration server by calling the callbacks defined by the Lamina behaviour on the config module, starts all the module's providers and loads all the configuration. If any step in this process fails the server will fail to start.","ref":"Lamina.Server.html#init/1","title":"Lamina.Server.init/1","type":"function"},{"doc":"","ref":"Lamina.Server.html#t:config_key/0","title":"Lamina.Server.config_key/0","type":"type"},{"doc":"","ref":"Lamina.Server.html#t:lifetime/0","title":"Lamina.Server.lifetime/0","type":"type"},{"doc":"","ref":"Lamina.Server.html#t:provider/0","title":"Lamina.Server.provider/0","type":"type"},{"doc":"","ref":"Lamina.Server.html#t:provider_state/0","title":"Lamina.Server.provider_state/0","type":"type"},{"doc":"","ref":"Lamina.Server.html#t:state/0","title":"Lamina.Server.state/0","type":"type"},{"doc":"","ref":"Lamina.Server.html#t:value/0","title":"Lamina.Server.value/0","type":"type"},{"doc":"Wrapper around access to a Lamina configuration module","ref":"Lamina.Server.ConfigModule.html","title":"Lamina.Server.ConfigModule","type":"module"},{"doc":"Call the Lamina callback on the configuration module to cast a configuration value.","ref":"Lamina.Server.ConfigModule.html#cast/3","title":"Lamina.Server.ConfigModule.cast/3","type":"function"},{"doc":"Call the config_change/3 callback on the configuration module.","ref":"Lamina.Server.ConfigModule.html#config_change/4","title":"Lamina.Server.ConfigModule.config_change/4","type":"function"},{"doc":"Call the Lamina callback on the configuration module and return a list of config keys.","ref":"Lamina.Server.ConfigModule.html#config_keys/1","title":"Lamina.Server.ConfigModule.config_keys/1","type":"function"},{"doc":"Ensure that the module in question actually implements the Lamina behaviour.","ref":"Lamina.Server.ConfigModule.html#is_lamina_module/1","title":"Lamina.Server.ConfigModule.is_lamina_module/1","type":"function"},{"doc":"Call the Lamina callback on the configuration module and return the providers.","ref":"Lamina.Server.ConfigModule.html#providers/1","title":"Lamina.Server.ConfigModule.providers/1","type":"function"},{"doc":"Call the Lamina callback on the configuration module to validate a configuration value.","ref":"Lamina.Server.ConfigModule.html#validate/3","title":"Lamina.Server.ConfigModule.validate/3","type":"function"},{"doc":"A wrapper around an individual configuration value. It makes the code much simpler. That is all.","ref":"Lamina.Server.ConfigValue.html","title":"Lamina.Server.ConfigValue","type":"module"},{"doc":"Perform a value cast using the ConfigValue's module.","ref":"Lamina.Server.ConfigValue.html#cast/1","title":"Lamina.Server.ConfigValue.cast/1","type":"function"},{"doc":"Has the ConfigValue expired?","ref":"Lamina.Server.ConfigValue.html#expired?/1","title":"Lamina.Server.ConfigValue.expired?/1","type":"function"},{"doc":"Initialise a new ConfigValue.","ref":"Lamina.Server.ConfigValue.html#init/6","title":"Lamina.Server.ConfigValue.init/6","type":"function"},{"doc":"Perform a value validation using the ConfigValue's module.","ref":"Lamina.Server.ConfigValue.html#validate/1","title":"Lamina.Server.ConfigValue.validate/1","type":"function"},{"doc":"","ref":"Lamina.Server.ConfigValue.html#t:config_key/0","title":"Lamina.Server.ConfigValue.config_key/0","type":"type"},{"doc":"","ref":"Lamina.Server.ConfigValue.html#t:lifetime/0","title":"Lamina.Server.ConfigValue.lifetime/0","type":"type"},{"doc":"","ref":"Lamina.Server.ConfigValue.html#t:provider/0","title":"Lamina.Server.ConfigValue.provider/0","type":"type"},{"doc":"","ref":"Lamina.Server.ConfigValue.html#t:provider_index/0","title":"Lamina.Server.ConfigValue.provider_index/0","type":"type"},{"doc":"","ref":"Lamina.Server.ConfigValue.html#t:t/0","title":"Lamina.Server.ConfigValue.t/0","type":"type"},{"doc":"Separates the functional implementation from the stateful GenServer.","ref":"Lamina.Server.Impl.html","title":"Lamina.Server.Impl","type":"module"},{"doc":"Query provider for the latest config_key . If the provider has a new value, and it is different to the old value, then broadcast the change to subscribers, and call the config module's config_change/3 callback.","ref":"Lamina.Server.Impl.html#get_latest_value/3","title":"Lamina.Server.Impl.get_latest_value/3","type":"function"},{"doc":"Initialise the table and server state from the provided options.","ref":"Lamina.Server.Impl.html#init/1","title":"Lamina.Server.Impl.init/1","type":"function"},{"doc":"If config_value is volatile, then ask the Lamina server to make sure that it has the freshest information.","ref":"Lamina.Server.Impl.html#maybe_get_latest/2","title":"Lamina.Server.Impl.maybe_get_latest/2","type":"function"},{"doc":"Refreshes all configuration for a provider.","ref":"Lamina.Server.Impl.html#refresh_provider/2","title":"Lamina.Server.Impl.refresh_provider/2","type":"function"},{"doc":"Call config_change/2 on all providers with a function which they can use to trigger refreshes.","ref":"Lamina.Server.Impl.html#set_provider_config_change_callbacks/1","title":"Lamina.Server.Impl.set_provider_config_change_callbacks/1","type":"function"},{"doc":"","ref":"Lamina.Server.Impl.html#t:config_key/0","title":"Lamina.Server.Impl.config_key/0","type":"type"},{"doc":"","ref":"Lamina.Server.Impl.html#t:provider/0","title":"Lamina.Server.Impl.provider/0","type":"type"},{"doc":"Helps the Lamina server deal with individual providers.","ref":"Lamina.Server.Provider.html","title":"Lamina.Server.Provider","type":"module"},{"doc":"","ref":"Lamina.Server.Provider.html#config_change/3","title":"Lamina.Server.Provider.config_change/3","type":"function"},{"doc":"","ref":"Lamina.Server.Provider.html#fetch_config/3","title":"Lamina.Server.Provider.fetch_config/3","type":"function"},{"doc":"","ref":"Lamina.Server.Provider.html#is_provider_module/1","title":"Lamina.Server.Provider.is_provider_module/1","type":"function"},{"doc":"","ref":"Lamina.Server.Provider.html#start/2","title":"Lamina.Server.Provider.start/2","type":"function"},{"doc":"","ref":"Lamina.Server.Provider.html#t:config_key/0","title":"Lamina.Server.Provider.config_key/0","type":"type"},{"doc":"","ref":"Lamina.Server.Provider.html#t:lifetime/0","title":"Lamina.Server.Provider.lifetime/0","type":"type"},{"doc":"","ref":"Lamina.Server.Provider.html#t:provider/0","title":"Lamina.Server.Provider.provider/0","type":"type"},{"doc":"","ref":"Lamina.Server.Provider.html#t:state/0","title":"Lamina.Server.Provider.state/0","type":"type"},{"doc":"Defines the state of the Lamina server process.","ref":"Lamina.Server.State.html","title":"Lamina.Server.State","type":"module"},{"doc":"","ref":"Lamina.Server.State.html#init/1","title":"Lamina.Server.State.init/1","type":"function"},{"doc":"","ref":"Lamina.Server.State.html#t:provider_state/0","title":"Lamina.Server.State.provider_state/0","type":"type"},{"doc":"","ref":"Lamina.Server.State.html#t:t/0","title":"Lamina.Server.State.t/0","type":"type"},{"doc":"A wrapper around ETS for our specific use cases.","ref":"Lamina.Server.Table.html","title":"Lamina.Server.Table","type":"module"},{"doc":"Delete any expired rows from the ETS table.","ref":"Lamina.Server.Table.html#expire/1","title":"Lamina.Server.Table.expire/1","type":"function"},{"doc":"Find the most likely configuration value for a given configuration key. This executes a gently complicated match spec against the ETS table to find a configuration value which has the highest provider priority and is not expired.","ref":"Lamina.Server.Table.html#get/2","title":"Lamina.Server.Table.get/2","type":"function"},{"doc":"Get a specific configuration value from the ETS table. Searches by config_key and provider only - does not take into account the row's expiry time .","ref":"Lamina.Server.Table.html#get/3","title":"Lamina.Server.Table.get/3","type":"function"},{"doc":"Insert any number of configuration values into the ETS table.","ref":"Lamina.Server.Table.html#insert/2","title":"Lamina.Server.Table.insert/2","type":"function"},{"doc":"Initialises a new ETS table.","ref":"Lamina.Server.Table.html#new/1","title":"Lamina.Server.Table.new/1","type":"function"},{"doc":"Remove matching configuration values from the ETS table.","ref":"Lamina.Server.Table.html#remove/3","title":"Lamina.Server.Table.remove/3","type":"function"},{"doc":"","ref":"Lamina.Server.Table.html#t:lifetime/0","title":"Lamina.Server.Table.lifetime/0","type":"type"},{"doc":"","ref":"Lamina.Server.Table.html#t:provider/0","title":"Lamina.Server.Table.provider/0","type":"type"},{"doc":"","ref":"Lamina.Server.Table.html#t:t/0","title":"Lamina.Server.Table.t/0","type":"type"},{"doc":"","ref":"Lamina.Server.Table.html#t:value/0","title":"Lamina.Server.Table.value/0","type":"type"},{"doc":"Lamina is unable to register the configuration module. Lamina uses an Elixir node-local Registry to keep track of all the configuration modules and their respective ETS tables. This error indicates that an attempt has been made to add the same configuration module twice. This is most likely to happen if you have accidentally added your configuration module to a supervisor more than once.","ref":"Lamina.Error.AlreadyRegisteredError.html","title":"Lamina.Error.AlreadyRegisteredError","type":"exception"},{"doc":"","ref":"Lamina.Error.AlreadyRegisteredError.html#t:t/0","title":"Lamina.Error.AlreadyRegisteredError.t/0","type":"type"},{"doc":"No configuration value was found. This is the result of querying the ETS table for a value, but no applicable record being present. Some possible reasons for this to happen: No providers returned a value for this config key. All values present in the table have expired. The config key is incorrect.","ref":"Lamina.Error.ConfigNotFoundError.html","title":"Lamina.Error.ConfigNotFoundError","type":"exception"},{"doc":"","ref":"Lamina.Error.ConfigNotFoundError.html#t:t/0","title":"Lamina.Error.ConfigNotFoundError.t/0","type":"type"},{"doc":"A configuration value returned by a provider has failed validation.","ref":"Lamina.Error.InvalidValueError.html","title":"Lamina.Error.InvalidValueError","type":"exception"},{"doc":"The module does not implement the Lamina behaviour. During startup, the Lamina server double checks that the module it's been asked to manage actually is a Lamina module and not just some other random module. Really the only way you should ever see this error in the wild is if you try and manually start Lamina.Server - which you probably shouldn't be doing anyway.","ref":"Lamina.Error.NotALaminaModuleError.html","title":"Lamina.Error.NotALaminaModuleError","type":"exception"},{"doc":"","ref":"Lamina.Error.NotALaminaModuleError.html#t:t/0","title":"Lamina.Error.NotALaminaModuleError.t/0","type":"type"},{"doc":"The module does not implement the Lamina.Provider behaviour. During startup, the Lamina server double checks that the providers it's been asked to use actually are Lamina.Provider modules and not just some other random module. Some reasons you could be seeing this error: You have misspelt the name of the module when using the provider macro in your configuration module (or you're missing an alias). You are developing a provider and haven't fully implemented all the required callbacks.","ref":"Lamina.Error.NotAProviderModuleError.html","title":"Lamina.Error.NotAProviderModuleError","type":"exception"},{"doc":"","ref":"Lamina.Error.NotAProviderModuleError.html#t:t/0","title":"Lamina.Error.NotAProviderModuleError.t/0","type":"type"},{"doc":"Lamina is unable to locate the configuration module in the registry. Lamina uses an Elixir node-local Registry to keep track of all the configuration modules and their respective ETS tables. This error indicates that a request has been made to retrieve the pid or table of a configuration module which is not present in the registry. This is most likely to happen if the Lamina server process has exited for some reason and could not be or has not yet been restarted.","ref":"Lamina.Error.NotRegisteredError.html","title":"Lamina.Error.NotRegisteredError","type":"exception"},{"doc":"","ref":"Lamina.Error.NotRegisteredError.html#t:t/0","title":"Lamina.Error.NotRegisteredError.t/0","type":"type"},{"doc":"The Lamina Server is in an invalid state. This is an internal error within Lamina.Server and is strongly indicitave of a bug. Please open an issue .","ref":"Lamina.Error.StateError.html","title":"Lamina.Error.StateError","type":"exception"},{"doc":"","ref":"Lamina.Error.StateError.html#t:t/0","title":"Lamina.Error.StateError.t/0","type":"type"}]